//  SettingsPanel.cpp
//
// 	This panel will get user settings. 
// 
//	russ 5/21/98
// 
//	Copyright 1997-1999, Be Incorporated.   All Rights Reserved.
//	This file may be used under the terms of the Be Sample Code License.

#include <string.h>
#include <stdlib.h>

#include <Alert.h>
#include <Box.h>
#include <Button.h>
#include <Screen.h>
#include <TextControl.h>
#include <Window.h>
#include <StringView.h>
#include <PopUpMenu.h>
#include <MenuField.h>
#include <MenuItem.h>
#include <CheckBox.h>

#include "Resource.h"
#include "SettingsPanel.h"
#include "IO.h"

const char* kSettingsBtnDone = "Done";
const char* kSettingsBtnCancel = "Cancel";
const char* kSettingsMsgCancel = "Cancel";
const char* kSettingsMsgOK = "OK";

static float
FontHeight(BView* target, bool full)
{
	font_height finfo;		
	target->GetFontHeight(&finfo);
	float h = finfo.ascent + finfo.descent;

	if (full)
		h += finfo.leading;
	
	return h;
}

static void CenterWindowOnScreen(BWindow* w)
{
	BRect	screenFrame = (BScreen(B_MAIN_SCREEN_ID).Frame());
	BPoint 	pt;
	pt.x = screenFrame.Width()/2 - w->Bounds().Width()/2;
	pt.y = screenFrame.Height()/2 - w->Bounds().Height()/2;

	if (screenFrame.Contains(pt))
		w->MoveTo(pt);
}

const int32 kPanelWidth = 300; 
const int32 kPanelHeight = 260;

SettingsPanel::SettingsPanel(BCallbackHandler* cb)
	:BWindow(BRect(0,0,kPanelWidth, kPanelHeight), "E-Drive Settings", B_MODAL_WINDOW,  
		B_NOT_RESIZABLE | B_NOT_ZOOMABLE | B_NOT_CLOSABLE)
{	
	m_CallBack = cb;	
	CreatePanelUI();
	
	Update(kFillControls);
	
	CenterWindowOnScreen(this);
	Show();      
} 

SettingsPanel::~SettingsPanel()
{
}

void SettingsPanel::MessageReceived(BMessage *msg) 
{
 		switch (msg->what){
			case MSG_DONE:
				m_CallBack->Done(B_NO_ERROR);
				PostMessage(B_QUIT_REQUESTED);
			break;
			
			case MSG_CANCEL:
				m_CallBack->Done(B_ERROR);
				PostMessage(B_QUIT_REQUESTED);
			break;
					
		default:
			BWindow::MessageReceived(msg);
			break;
		}
}

void SettingsPanel::Update(int16 direction)
{		
    // Load up the controls
    if (direction == kFillControls){ 
   		m_vpiCtl->SetValue(m_vpi);
   		m_vciCtl->SetValue(m_vci);
		m_showPacketsCtl->SetValue(m_showPackets);
	}
    else{
        // Fill members from controls
		m_vpi = m_vpiCtl->Value();
		m_vci = m_vciCtl->Value();
		m_showPackets = m_showPacketsCtl->Value();
    }
}

void SettingsPanel::CreatePanelUI() 
{
	BTextView* tv;
	
	BRect rect(Bounds());
	rect.InsetBy(-1.0, -1.0);
	m_MainBG = new BBox(rect);
	m_MainBG->SetFont(be_plain_font);
	AddChild(m_MainBG);
	
	rect.left = 11; rect.right = Bounds().Width() - 11;
	rect.top = 13; rect.bottom = rect.top + FontHeight(m_MainBG,true)+5;
	m_Label = new BStringView(rect, "", "");
	m_Label->SetFont(be_bold_font);
	m_Label->SetText("Settings for testing ethernet drivers.");
	m_MainBG->AddChild(m_Label);
	
	// Reset this here so that stringwidths can be calculated for plain.
	m_MainBG->SetFont(be_plain_font);
	
	rect.top = m_Label->Frame().bottom + 10;
	rect.bottom = rect.top + 1;
	m_vpiCtl = new BTextControl(rect, "", "vpi:", "", NULL);
	m_vpiCtl->SetDivider(m_MainBG->StringWidth("vpi:") + 10);
	m_vpiCtl->SetAlignment(B_ALIGN_RIGHT, B_ALIGN_LEFT);
	m_vpiCtl->MakeFocus(true);
	tv = m_vpiCtl->TextView();
	if (tv)
		tv->SetMaxBytes(128);
	m_MainBG->AddChild(m_vpiCtl);
	
	rect.top = m_vpiCtl->Frame().bottom + 10;
	rect.bottom = rect.top + 1;
	m_vciCtl = new BTextControl(rect, "", "vci:", "", NULL);
	m_vciCtl->SetDivider(m_MainBG->StringWidth("vci:") + 10);
	m_vciCtl->SetAlignment(B_ALIGN_RIGHT, B_ALIGN_LEFT);
	tv = m_vciCtl->TextView();
	if (tv)
		tv->SetMaxBytes(128);
	m_MainBG->AddChild(m_vciCtl);
	
	
	rect.top = m_vciCtl->Frame().bottom + 10;
	rect.bottom = rect.top + 1;
	rect.left = 11;
	rect.right = Bounds().Width() - 10;	
	m_showPacketsCtl = new BCheckBox(rect, "", "Display packets", 
									new BMessage(999));
	m_MainBG->AddChild(m_showPacketsCtl);
	
	/////////////// Buttons ////////////////	
	rect.right = Bounds().Width() - 10;
	rect.left = rect.right - 75;
	rect.top = m_showPacketsCtl->Frame().bottom + 15;
	rect.bottom = rect.top + 1;
	m_DoneBtn = new BButton(rect, "", kSettingsBtnDone, new BMessage(MSG_DONE),
			B_FOLLOW_BOTTOM | B_FOLLOW_RIGHT);
	
	rect.right = rect.left - 10;
	rect.left = rect.right - 75;
	m_CancelBtn = new BButton(rect, "", kSettingsBtnCancel, new BMessage(MSG_CANCEL),
			B_FOLLOW_BOTTOM | B_FOLLOW_RIGHT);
	m_MainBG->AddChild(m_CancelBtn);	

	m_MainBG->AddChild(m_DoneBtn);
	SetDefaultButton(m_DoneBtn);
}

void SettingsPanel::SetVpi(uint16 vpi)
{	
	if (Lock()){
		m_vpi = vpi;	
		Update(kFillControls);
		m_vpiCtl->MakeFocus(true);		    		  
    	Unlock();
    }
}


void SettingsPanel::SetVci(uint16 vci)
{	
	if (Lock()){
		m_vci = vci;	
		Update(kFillControls);
		m_vciCtl->MakeFocus(true);		    		  
    	Unlock();
    }
}


void SettingsPanel::SetShowPackets(const bool showPackets)
{	
	if (Lock()){
		m_showPackets = showPackets;		
		Update(kFillControls);	    		  
    	Unlock();
    }
}


void SettingsPanel::GetVpi(uint16 *vpi)
{		
	Update(kEmptyControls);
	*vpi = m_vpi;
}

void SettingsPanel::GetVci(uint16 *vci)
{		
	Update(kEmptyControls);
	*vci = m_vci;
}


void SettingsPanel::GetShowPackets(bool* showPackets)
{		
	Update(kEmptyControls);
	*(bool*)showPackets = m_showPackets;
}

