#ifndef DRIVER_H
#define DRIVER_H

#define DRIVER_NAME	"speed_touch"

#define	LOGID	"\033[34m" DRIVER_NAME "\033[m: "
#define	ERR		"\033[31merror\033[m: "
#define	WARN	"\033[31mwarning\033[m: "
#define	assert(x) \
	((x) ? 0 : dprintf (LOGID ERR "assertion failed at " __FILE__ ", line %d\n", __LINE__))


#define DEVICE_NAME 			DRIVER_NAME
#define DEVICE_NAME_LENGTH		(64)			
#define DEVICE_PATH 			"net/usb/" DRIVER_NAME "/%d"

#define STUSB_TIMEOUT		(1000000)		// 1 second

typedef struct
	{
	uint8	 byte[6];
	}
mac_address_t;

typedef struct device device;

struct device {
	device *					next;				// linked list
	
	long						id;
	char 						name[DEVICE_NAME_LENGTH + 1];
	uint32						flags;
	uint32                      write_on;
	uint32						debug_level;

	// ---- Locks mechanisms
	uint32						open_count;
	
	sem_id						status_semaphore;
	sem_id						read_semaphore;
	sem_id						write_semaphore;
	sem_id                      pdu_read_semaphore;	
	
	// threads
	thread_id                   pollingTid;	

	// ---- device data
	const usb_device *			ud;						// our usb device

	const usb_interface_info *	firmware_interface;
	const usb_endpoint_info *	firmware_in_endpoint;	// Firmware bulk in endpoint
	const usb_endpoint_info *	firmware_out_endpoint;	// Firmware bulk in endpoint
	
	const usb_interface_info *	status_interface;
	const usb_endpoint_info *	status_int_endpoint;	// Status interrupt endpoint
	
	const usb_interface_info *	data_interface;
	const usb_endpoint_info *	data_in_endpoint;		// Data bulk in endpoint
	const usb_endpoint_info *	data_out_endpoint;		// Data bulk out endpoint
	
	// ---- device buffers
	void *						buffer;					// Common buffer (for interrupt, bulk in and bukl out)
	area_id						buffer_area_id;			// Common buffer area id

	void *						status_buffer;			// Address of status data in common buffer
	void *						write_buffer;			// Address of write data in common buffer
	uint32						write_status;
	uint32						write_actual_length;	
	void *						read_buffer;			// Address of read data in common buffer
	uint32						read_status;
	uint32						read_actual_length;
	uint32						unread_bytes;

	// ---- ATM parameters
	uint16                      vpi;
	uint16                      vci;
	// ---- device states
	mac_address_t				mac_address;	// MAC address
	bool						up;
	bool						pollingOn;
	// ---- Somes statistics
	volatile					nb_bytes_sent;
	volatile					nb_bytes_received;
	volatile					nb_request_failure;
	volatile					nb_write_errors;
	volatile					nb_read_errors;
	volatile                    us_bitrate;
	volatile                    ds_bitrate;
	volatile                    tx_cells;
	volatile                    rx_cells;
	volatile                    tx_dropped_cells;
	volatile                    rx_dropped_cells;
	volatile                    drc;
	volatile					urc;
	volatile					alarms;
	volatile					line_status;
};

status_t	usb_read(device * dev, void * data, size_t sz, usb_pipe * handle, bigtime_t	timeout); // handle = NULL -> default data in
status_t	usb_write(device * dev, void * data, size_t sz, usb_pipe * handle, bigtime_t timeout); // handle = NULL -> default data out


#endif	// DRIVER_H
