#include "aal5_packet.h"
#include "crc32.h"
#include <string.h>
#include <malloc.h>

extern FILE *f;

unsigned int calc_crc(unsigned char *mem, int len, unsigned int initial)
{
	unsigned int crc;

	crc = initial;
	for(; len; mem++, len--)
		crc = CRC32(*mem, crc);
	return (crc);
}

AAL5Packet::AAL5Packet(void) {
	size = 0;
	data = malloc(1);
	base = 0;
	fprintf(f,"AAL5Packet::AAL5Packet(void) ppp pdu length=%d\n",(int)size);
	fflush(f);
}

AAL5Packet::AAL5Packet(BNetPacket *packet) : BNetPacket() {
	uint16 ppp_pdu_len;
	uint8 *aal5_buf;
	
	base = 0;
	size = packet->Size();
	data = malloc(size);
	// retrive data
	copy_packet(packet, 0, this, 0, packet->Size());
	// remove padding and trailer	
	aal5_buf = (uint8 *)data;
	ppp_pdu_len = (aal5_buf[size - 6] << 8) | aal5_buf[size - 5];
	fprintf(f,"AAL5Packet::AAL5Packet(packet) ppp pdu length=%d cpcs pdu len=%d\n",ppp_pdu_len,(int)size);
	size = ppp_pdu_len;
	fflush(f);
}


void AAL5Packet::SetData(const void *data,size_t length,uint16 offset) {
	if (length > 1522)
		return;
	SetSize(length+offset);
	Write(offset,(char *)data,length);
}

size_t AAL5Packet::GetData(const void *buffer,size_t length,uint16 offset) {
	size_t size = this->size  - offset;
	size = (length > size) ? size : length;
	Read(offset,(char *)buffer,size);
	return size;
}


void AAL5Packet::Finalize(void) {
	int      cpcs_pdu_len;
	uint16   ppp_pdu_len;
	int      pad_len;
	uint16   offset;
	uint32   crc32;
	uint8    crc_bytes[4];
	
	// AAL5 encapsulation	
	offset = ppp_pdu_len = size;
	cpcs_pdu_len = 48 * ((ppp_pdu_len + 8 + 47) / 48);
	pad_len      = cpcs_pdu_len - ppp_pdu_len - 8;
	fprintf(f,"AAL5Packet::Finalize() AAL5 pdu length=%d pad=%d\n",cpcs_pdu_len, pad_len);
	fflush(f);
	// resize frame
	SetSize(cpcs_pdu_len);
	if (pad_len != 0) {
		uint8 *padding = (uint8 *)malloc(pad_len + 2); // UU + CPI = 2 bytes
		memset(padding, 0, pad_len + 2);
		Write(offset,(char *)padding,pad_len + 2);
		offset += (pad_len + 2);
		free(padding);
	}
	// UU and CPI are already set to 0
	// Payload size
	ppp_pdu_len = htons(ppp_pdu_len);
	Write(offset,(char *)&ppp_pdu_len,2);
	offset += 2;
	// CRC32
	crc32 = ~calc_crc((uint8 *)data, cpcs_pdu_len-4, 0xffffffff);
	fprintf(f,"AAL5Packet::Finalize() crc32=%x\n",(int)crc32);
	fflush(f);
	crc32 = htonl(crc32);
	crc_bytes[0] = (crc32 & 0x00ff);      /* least significant byte first */ 
	crc_bytes[1] = ((crc32 >>= 8) & 0x00ff); 
	crc_bytes[2] = ((crc32 >>= 8) & 0x00ff); 
	crc_bytes[3] = ((crc32 >> 8) & 0x00ff);
	Write(offset,(char *)crc_bytes,4);
}

unsigned AAL5Packet::Size(void) {
	return size - base;
}

void AAL5Packet::SetSize(unsigned size) {
	unsigned int oldsize;
	
	if (this->size == size + base)
		return;
	oldsize = this->size;
	this->size = size + base;
	if(oldsize < this->size)
		data = realloc(data,this->size);
}

unsigned AAL5Packet::Base(void) {
	return base;
}

void AAL5Packet::SetBase(int offset) {
	base = offset;
}

char *AAL5Packet::DataBlock(unsigned offset, unsigned *size) {
	void *to_return;
	*size = ((this->size - base - offset) > *size) ? *size : (this->size - base - offset);
	to_return = (void *)(uint32(data) + base + offset);
	if ((this->size - base - offset) <= 0) {
		*size = 1 /* never return 0 */;
		return NULL;
	}
	return (char *)to_return;
}
	

AAL5Packet::~AAL5Packet(void) {
	free(data);
}